@@ -1,12 +1,25 @@
+.return_zerorow <- function(x, row_header, .name_repair) {
+    jcol <- ifelse(row_header, 2, 1)
+    col_n <- vctrs::vec_as_names(as.character(x[1,jcol:ncol(x)]), repair = .name_repair)
+    g <- data.frame(matrix(data = NA_character_, ncol = length(col_n), nrow = 0))
+    colnames(g) <- col_n
+    return(g)
+}
+
 .change_df_with_col_row_header <- function(x, col_header, row_header, .name_repair) {
-    if((nrow(x) < 2 && col_header )|| (ncol(x) < 2 && row_header)) {
+    if (nrow(x) == 1 && col_header && isFALSE(getOption("readODS.v200", FALSE))) {
+        return(.return_zerorow(x, row_header, .name_repair))
+    }
+    if (((nrow(x) < 2 && col_header ) || (ncol(x) < 2 && row_header)) &&
+        isTRUE(getOption("readODS.v200", FALSE))) {
+        ## 2.0.0 behavior
         warning("Cannot make column/row names if this would cause the dataframe to be empty.", call. = FALSE)
         return(x)
     }
     irow <- ifelse(col_header, 2, 1)
     jcol <- ifelse(row_header, 2, 1)
 
-    g <- x[irow:nrow(x), jcol:ncol(x), drop=FALSE] # maintain as dataframe for single column
+    g <- x[irow:nrow(x), jcol:ncol(x), drop = FALSE] # maintain as dataframe for single column
 
 
     rownames(g) <- if(row_header) x[seq(irow, nrow(x)), 1] else NULL # don't want character row headers given by 1:nrow(g)
@@ -19,8 +32,10 @@
     return(g)
 }
 
+
+
 ## Based on readxl, although the implementation is different.
-## If max row is -1, read to end of row. 
+## If max row is -1, read to end of row.
 ## Row and column-numbers are 1-based
 .standardise_limits <- function(range, skip) {
     if(is.null(range)) {
@@ -50,17 +65,6 @@
     return(limits)
 }
 
-.silent_type_convert <- function(x, verbose = TRUE, na = c("", "NA")) {
-    if (verbose) {
-        res <- readr::type_convert(df = x, na = na)
-    } else {
-        suppressMessages({
-            res <- readr::type_convert(df = x, na = na)
-        })
-    }
-    return(res)
-}
-
 .convert_strings_to_factors <- function(df) {
     i <- vapply(df, is.character, logical(1))
     df[i] <- lapply(df[i], as.factor)
@@ -106,6 +110,69 @@
     if (row_names && as_tibble) {
         stop("Tibbles do not support row names. To use row names, set as_tibble to false", call. = FALSE)
     }
+    if (!inherits(col_types, "col_spec") &&
+        isFALSE(is.na(col_types)) &&
+        isFALSE(is.null(col_types)) &&
+        isFALSE(is.character(col_types)) &&
+        isFALSE(is.list(col_types))) {
+        stop("Unknown col_types. Can either be a class col_spec, list, character, NULL or NA.",
+             call. = FALSE)
+    }
+}
+
+.return_empty <- function(as_tibble = FALSE) {
+    if (getOption("readODS.v200", FALSE)) {
+        warning("empty sheet, return empty data frame.", call. = FALSE)
+    }
+    if (as_tibble) {
+        return(tibble::tibble())
+    }
+    return(data.frame())
+}
+
+.type_convert <- function(df, col_types = NULL, verbose = TRUE, na = c("", "NA")) {
+    if (verbose) {
+        res <- readr::type_convert(df = df, col_types, na = na)
+    } else {
+        suppressMessages({
+            res <- readr::type_convert(df = df, col_types, na = na)
+        })
+    }
+    return(res)
+}
+
+.handle_col_types <- function(res, col_types, verbose, na) {
+    if (isTRUE(is.na(col_types)) || nrow(res) == 0) {
+        return(res)
+    }
+    .type_convert(df = res, col_types = col_types, verbose = verbose, na = na)
+}
+
+## standardise `sheet` parameter as a number, i.e. sheet_index
+.standardise_sheet <- function(sheet, sheets, limits, range) {
+    sheet_from_range <- cellranger::as.cell_limits(range)[["sheet"]]
+    if (!is.null(range) && !is.na(sheet_from_range)) {
+        if (sheet != 1) {
+            warning("Sheet suggested in range and using sheet argument. Defaulting to range",
+                    call. = FALSE)
+        }
+        sheet <- sheet_from_range ## override
+    }
+    is_in_sheet_names <- stringi::stri_cmp(e1 = sheet, e2 = sheets) == 0
+    if (!is.numeric(sheet) && !any(is_in_sheet_names)) {
+        stop(paste0("No sheet found with name '", sheet, "'", sep = ""),
+             call. = FALSE)
+    }
+    if (is.numeric(sheet) && sheet > length(sheets)) {
+        stop(paste0("File contains only ", length(sheets), " sheets. Sheet index out of range.",
+                    call. = FALSE))
+    }
+    if (!is.numeric(sheet)) {
+        sheet_index <- which(is_in_sheet_names)
+    } else {
+        sheet_index <- sheet
+    }
+    return(sheet_index)
 }
 
 .read_ods <- function(path,
@@ -134,65 +201,33 @@
         strings_as_factors,
         verbose,
         as_tibble)
-    # Get cell range info
-    limits <- .standardise_limits(range, skip)
-    # Get sheet number.
+    path <- normalizePath(path)
     if (flat) {
-        sheets <- get_flat_sheet_names_(file = path, include_external_data = TRUE)
+        .get_sheet_names_func <- get_flat_sheet_names_
+        .read_ods_func <- read_flat_ods_
     } else {
-        sheets <- get_sheet_names_(file = path, include_external_data = TRUE)
-    }
-    sheet_name <- cellranger::as.cell_limits(range)[["sheet"]]
-    if(!is.null(range) && !is.na(sheet_name)) {
-        if(sheet != 1) {
-            warning("Sheet suggested in range and using sheet argument. Defaulting to range",
-                call. = FALSE)
-        }
-        is_in_sheet_names <- stringi::stri_cmp(e1 = sheet_name, e2 = sheets) == 0
-        if(any(is_in_sheet_names)) {
-            sheet <- which(is_in_sheet_names)
-        } else {
-            stop(paste0("No sheet found with name '", sheet_name, "'", sep = ""),
-                call. = FALSE)
-        }
-    } else {
-        is_in_sheet_names <- stringi::stri_cmp(e1 = sheet, e2 = sheets) == 0
-        if (!is.numeric(sheet) && any(is_in_sheet_names)) {
-            sheet <- which(is_in_sheet_names)
-        } else if (!is.numeric(sheet)) {
-            stop(paste0("No sheet found with name '", sheet, "'", sep = ""), 
-                call. = FALSE)
-        }
-        if (sheet > length(sheets)) {
-            stop(paste0("File contains only ", length(sheets), " sheets. Sheet index out of range.",
-                call. = FALSE))
-        }
+        .get_sheet_names_func <- get_sheet_names_
+        .read_ods_func <- read_ods_
     }
+    ## Get cell range info
+    limits <- .standardise_limits(range, skip)
+    sheet_index <- .standardise_sheet(sheet = sheet, sheets = .get_sheet_names_func(file = path, include_external_data = TRUE),
+                                      limits = limits, range = range)
+    strings <- .read_ods_func(file = path,
+                              start_row = limits["min_row"],
+                              stop_row = limits["max_row"],
+                              start_col = limits["min_col"],
+                              stop_col = limits["max_col"],
+                              sheet_index = sheet_index,
+                              formula_as_formula = formula_as_formula)
 
-    if(flat) {
-        strings <- read_flat_ods_(file = path,
-                                  start_row = limits["min_row"],
-                                  stop_row = limits["max_row"],
-                                  start_col = limits["min_col"],
-                                  stop_col = limits["max_col"],
-                                  sheet = sheet,
-                                  formula_as_formula = formula_as_formula)
-    } else {
-        strings <- read_ods_(file = path,
-                             start_row = limits["min_row"],
-                             stop_row = limits["max_row"],
-                             start_col = limits["min_col"],
-                             stop_col = limits["max_col"],
-                             sheet = sheet,
-                             formula_as_formula = formula_as_formula)
+    if (((strings[1] == 0 || strings[2] == 0)) &&
+        isTRUE(getOption("readODS.v200", FALSE))) {
+        return(.return_empty(as_tibble = as_tibble))
     }
-    if(strings[1] == 0 || strings[2] == 0) {
-        warning("empty sheet, return empty data frame.", call. = FALSE)
-        if(as_tibble) {
-            return(tibble::tibble())
-        } else {
-            return(data.frame())
-        }
+    if (((strings[1] == 0 || strings[2] == 0) || (strings[1] == 1 && row_names)) &&
+        isFALSE(getOption("readODS.v200", FALSE))) {
+        return(.return_empty(as_tibble = as_tibble))
     }
     res <- as.data.frame(
         matrix(
@@ -201,27 +236,14 @@
             byrow = TRUE),
         stringsAsFactors = FALSE)
     res <- .change_df_with_col_row_header(x = res, col_header = col_names, row_header = row_names, .name_repair = .name_repair)
-    res <- data.frame(res)
-    if (inherits(col_types, 'col_spec')) {
-        res <- readr::type_convert(df = res, col_types = col_types, na = na)
-    } else if (length(col_types) == 0 && is.null(col_types)) {
-        res <- .silent_type_convert(x = res, verbose = verbose, na = na)
-    } else if (length(col_types) == 1 && is.na(col_types[1])) {
-        {} #Pass
-    } else {
-        stop("Unknown col_types. Can either be a class col_spec, NULL or NA.",
-            call. = FALSE)
-    }
-
+    res <- .handle_col_types(data.frame(res), col_types = col_types, verbose = verbose, na = na)
     if (strings_as_factors) {
         res <- .convert_strings_to_factors(df = res)
     }
-
     if (as_tibble) {
         res <- tibble::as_tibble(x = res, .name_repair = .name_repair)
     }
     return(res)
-
 }
 
 #' Read Data From (F)ODS File
@@ -232,12 +254,12 @@
 #' @param path path to the (f)ods file.
 #' @param sheet sheet to read. Either a string (the sheet name), or an integer sheet number. The default is 1.
 #' @param col_names logical, indicating whether the file contains the names of the variables as its first line. Default is TRUE.
-#' @param col_types Either NULL to guess from the spreadsheet or refer to [readr::type_convert()] to specify cols specification. NA will return a data frame with all columns being "characters".
+#' @param col_types Either NULL to guess from the spreadsheet or refer to [readr::type_convert()] to specify cols specification. It can also be a shorthand such as "ccf" ("character", "character", "factor"), a list, or an object created by [readr::cols()]. NA will return a data frame with all columns being "characters". Please note that it will not speed up the reading by a lot by specifying this parameter explicitly. It is more for accuracy.
 #' @param na Character vector of strings to use for missing values. By default read_ods converts blank cells to missing data. It can also be set to
 #' NULL, so that empty cells are treated as NA.
 #' @param skip the number of lines of the data file to skip before beginning to read data. If this parameter is larger than the total number of lines in the ods file, an empty data frame is returned.
 #' @param formula_as_formula logical, a switch to display formulas as formulas "SUM(A1:A3)" or as the resulting value "3"... or "8".. . Default is FALSE.
-#' @param range selection of rectangle using Excel-like cell range, such as \code{range = "D12:F15"} or \code{range = "R1C12:R6C15"}. Cell range processing is handled by the \code{\link[=cellranger]{cellranger}} package.
+#' @param range selection of rectangle using Excel-like cell range, such as \code{range = "D12:F15"} or \code{range = "R1C12:R6C15"}. Cell range processing is handled by the \code{\link[=cellranger]{cellranger}} package. If sheet name is in the range, such as \code{range = "Sheet2!A2:B7"}, this sheet name is used instead of the provided `sheet`. If `sheet` is not the default value (1), a warning is given.
 #' @param row_names logical, indicating whether the file contains the names of the rows as its first column. Default is FALSE.
 #' @param strings_as_factors logical, if character columns to be converted to factors. Default is FALSE.
 #' @param verbose logical, if messages should be displayed. Default is FALSE.
@@ -248,9 +270,9 @@
 #'  - `"check_unique"`: Check names are unique, but do not repair
 #'  - `"universal"` : Checks names are unique and valid R variables names in scope
 #'  - A function to apply custom name repair.
-#'  
+#'
 #'  Default is `"unique"`.
-#'  
+#'
 #' @return A tibble (\code{tibble}) or data frame (\code{data.frame}) containing a representation of data in the (f)ods file.
 #' @author Peter Brohan <peter.brohan+cran@@gmail.com>, Chung-hong Chan <chainsawtiney@@gmail.com>, Gerrit-Jan Schutten <phonixor@@gmail.com>
 #' @examples
@@ -267,6 +289,12 @@
 #' read_fods("starwars.fods", sheet = 2)
 #' # Read a specific range, e.g. A1:C11
 #' read_fods("starwars.fods", sheet = 2, range = "A1:C11")
+#' # Give a warning and read from Sheet1 (not 2)
+#' read_fods("starwars.fods", sheet = 2, range = "Sheet1!A1:C11")
+#' # Specifying col_types as shorthand, the third column as factor; other by guessing
+#' read_ods("starwars.ods", col_types = "??f")
+#' # Specifying col_types as list
+#' read_ods("starwars.ods", col_types = list(species = "f"))
 #' }
 #' @export
 read_ods <- function(path,
@@ -315,7 +343,7 @@ read_fods <- function(path,
                         as_tibble = TRUE,
                         .name_repair = "unique") {
     ## Should use match.call but there's a weird bug if one of the variable names is 'file'
-    .read_ods(path = path,
+    .read_ods(path = normalizePath(path, mustWork = FALSE),
               sheet = sheet,
               col_names = col_names,
               col_types = col_types,
